/*
 * Support for aliased pointers
 *
 * Copyright (C) 1998-2001 Jose' Orlando Pereira, Universidade do Minho
 */
/*
 * XTL2 - the eXternalization Template Library v. 2
 * http://xtl.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 *
 * $Id: aliased.h,v 1.4 2001/01/04 11:12:53 jop Exp $
 */


#ifndef __XTL2_ALIASED_H
#define __XTL2_ALIASED_H

#include <map.h>

namespace xtl {

// adaptors

template <class Adaptor, class Bucket>
struct _aliased_adaptor {
	typedef typename Adaptor::type type;
	typedef typename Bucket::index index;

	Adaptor adaptor;
	Bucket& bucket;

	_aliased_adaptor(Adaptor& a, Bucket& b):adaptor(a),bucket(b) {}

	template <class Visitor>
	void composite(Visitor& visitor, destructor* kind) {
		const int* p=bucket.find(adaptor.ptr);
		if (p==NULL) {
			bool lead=true;
			visitor.visit(lead);
			visit(visitor, kind, adaptor);
			bucket.insert(adaptor.ptr);
		} else {
			bool lead=false;
			index seq=*p;
			visitor.visit(lead);
			visitor.visit(seq);
		}
	}

	template <class Visitor>
	void composite(Visitor& visitor, constructor* kind) {
		bool lead;
		visitor.visit(lead);
		if (lead) {
			visit(visitor, kind, adaptor);
			bucket.insert(adaptor.ptr);
		} else {
			index seq;
			visitor.visit(seq);
			adaptor.ptr=bucket.find(seq);
		}
	}
};

template <class Adaptor, class Bucket>
inline _aliased_adaptor<Adaptor,Bucket> aliased(Adaptor a, Bucket& b) {
	return _aliased_adaptor<Adaptor,Bucket>(a, b);
}

template <class Type, class Idx>
class destructor_bucket {
 private:
	typedef typename std::map<Type*,Idx> data_t;
	data_t data;
	Idx cnt;

 public:
 	typedef Idx index;

	destructor_bucket():cnt(0) {}

	Idx const* find(Type* ptr) {
		typename data_t::iterator i=data.find(ptr);
		if (i==data.end())
			return NULL;
		else
			return &(*i).second;
	}

	void insert(Type* ptr) {
		data.insert(make_pair(ptr,cnt++));
	}
};

template <class Type, class Idx>
class constructor_bucket {
 private:
	typedef typename std::map<Idx,Type*> data_t;
	data_t data;
	Idx cnt;

 public:
 	typedef Idx index;

	constructor_bucket():cnt(0) {}

	Type* find(Idx idx) {
		typename data_t::iterator i=data.find(idx);
		return (*i).second;
	}

	void insert(Type* ptr) {
		data.insert(make_pair(cnt++,ptr));
	}
};

};

#endif

