/*
 * Buffer adaptors for binary formats
 *
 * Copyright (C) 1998-2001 Jose' Orlando Pereira, Universidade do Minho
 * Copyright (C) 2000 Lutz Latta, lutz@latta.de
 */
/*
 * XTL2 - the eXternalization Template Library v. 2
 * http://xtl.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 *
 * $Id: buffers.h,v 1.4 2001/01/04 11:12:53 jop Exp $
 */

/*
 * Copyright (C) 1998-2001 jop
 * Copyright (C) 2000 Lutz Latta, lutz@latta.de
 */

#ifndef __XTL2_BUFFERS_H
#define __XTL2_BUFFERS_H

#include <typeinfo>
#include <stdexcept>

namespace xtl {

// simple memory buffer

class buffer {
 private:
 	char *data, *ptr, *lim;

 public:
	class buffer_overflow_error: public std::overflow_error {
	 public:
	 	int bytes_left;
		int bytes_needed;

		buffer_overflow_error(int left, int needed):
		std::overflow_error("xtl::buffer overflow"),
			bytes_left(left),bytes_needed(needed) {}
	};

 	buffer(void *buf, int size):
		data((char*)buf),ptr((char*)buf),lim(((char*)buf)+size) {}

	inline void read(char* ptr, int size)
		{std::memcpy(ptr, require(size), size);}
	inline void write(char const* ptr, int size)
		{std::memcpy(require(size), ptr, size);}

	inline void* require(int size) {
		char* aux=ptr;
		if ((ptr+=size)>lim)
			throw buffer_overflow_error(lim-aux,size);
		return aux;
	}

	inline void flush(int size) {}

	inline int size() {return ptr-data;}
};

template <class Stream>
class _istream_adaptor {
 private:
	Stream& stream;
	char buffer[8];

 public:
	_istream_adaptor(Stream& str): stream(str) {}

	inline void* require(int size) {
		read(buffer, size);
		return buffer;
	}

	inline void read(char* ptr, int size) {
		stream.read(ptr, size);
		//if (stream.fail())
		//	abort();
			//throw std::out_of_range("xtl::_istream_adaptor: read error");
	}

};

template <class Stream>
class _ostream_adaptor {
 private:
	Stream& stream;
	char buffer[8];

 public:
	_ostream_adaptor(std::ostream &str): stream(str) {}

	inline void* require(int size) {
		return buffer;
	}

	inline void flush(int size) {
		write(buffer, size);
	}

	inline void write(char const* ptr, int size) {
		stream.write(ptr, size);
		if (stream.fail())
			abort();
			//throw std::out_of_range("xtl::_ostream_adaptor: write error");
	}
};

};

#endif
