/*
 * Binary formats
 *
 * Copyright (C) 1998-2001 Jose' Orlando Pereira, Universidade do Minho
 * Copyright (C) 2000 Angus Leeming, a.leeming@ic.ac.uk
 */
/*
 * XTL2 - the eXternalization Template Library v. 2
 * http://xtl.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 *
 * $Id: formats.h,v 1.6 2001/01/04 11:12:53 jop Exp $
 */


#ifndef __XTL2_FORMATS_H
#define __XTL2_FORMATS_H

namespace xtl {

template <class Buffer>
class _XDR_output {
 private:
	Buffer& buf;
	typedef int word;

#if (__BYTE_ORDER == __LITTLE_ENDIAN)

	template <class Type>
	void output_word(Type const& data) {
		*reinterpret_cast<word*>(buf.require(4))=
			bswap_32(static_cast<word>(data)); 
		buf.flush(4);
	} 

	template <class Type>
	void output_hyper(Type const& data) {
		union { Type ll; word i[2]; } store;
		store.ll=data;
		*reinterpret_cast<word*>(buf.require(4))=bswap_32(store.i[1]);
		buf.flush(4);
		*reinterpret_cast<word*>(buf.require(4))=bswap_32(store.i[0]);
		buf.flush(4);
	} 

#elif (__BYTE_ORDER == __BIG_ENDIAN)

	template <class Type>
	void output_word(Type const& data) {
		*reinterpred_cast<word*>(buf.require(4))=
			static_cast<word>(data); 
		buf.flush(4);
	} 

	template <class Type>
	void output_hyper(Type const& data) {
		union { Type ll; word i[2]; } store;
		store.ll=data;
		*reinterpret_cast<word*>(buf.require(4))=store.i[0];
		buf.flush(4);
		*reinterpret_cast<word*>(buf.require(4))=store.i[1];
		buf.flush(4);
	} 

#endif

 public:
 	typedef destructor kind;

	_XDR_output(Buffer& buf):buf(buf) {}

	template <class Type>
	void visit(Type const& data)
		{output_word(data);}

	void visit(float const& data)
		{output_word(*reinterpret_cast<word const*>(&data));}

	void visit(longlong const& data)
		{output_hyper(data);}
	void visit(unsignedlonglong const& data)
		{output_hyper(data);}
	void visit(double const& data)
		{output_hyper(*reinterpret_cast<longlong const*>(&data));}

	template <class CharSet>
	void visit(char const* data, int size, CharSet) {
		buf.write(data, size);
		if (size%4!=0) {
			int align=4-size%4;
			std::memset(buf.require(align), 0, align);
			buf.flush(align);
		}
	}
};

template <class Stream>
class XDR_output: public _XDR_output<_ostream_adaptor<Stream> > {
 private:
 	_ostream_adaptor<Stream> adaptor;

 public:
 	XDR_output(Stream& str):adaptor(str),
		_XDR_output<_ostream_adaptor<Stream> >(adaptor) {}
};
  
template <>
class XDR_output<buffer>: public _XDR_output<buffer> {
 public:
 	XDR_output<buffer>(buffer& buf):_XDR_output<buffer>(buf) {}
};
  
template <class Buffer>
class _XDR_input {
 private:
	Buffer& buf;
	typedef int word;

#if (__BYTE_ORDER == __LITTLE_ENDIAN)

	template <class Type>
	void input_word(Type& data) {
		data=static_cast<Type>(bswap_32(*reinterpret_cast<word*>(buf.require(4))));
	} 

	template <class Type>
	void input_hyper(Type& data) {
		union { Type ll; word i[2]; } store;
		store.i[1]=bswap_32(*reinterpret_cast<word*>(buf.require(4)));
		store.i[0]=bswap_32(*reinterpret_cast<word*>(buf.require(4)));
		data=store.ll;
	} 

#elif (__BYTE_ORDER == __BIG_ENDIAN)

	template <class Type>
	void input_word(Type& data) {
		data=static_cast<Type>(*reinterpret_cast<word*>(buf.require(4)));
	} 

	template <class Type>
	void input_hyper(Type& data) {
		union { Type ll; word i[2]; } store;
		store.i[1]=*reinterpret_cast<word*>(buf.require(4));
		store.i[0]=*reinterpret_cast<word*>(buf.require(4));
		data=store.ll;
	} 

#endif

 public:
 	typedef constructor kind;

	_XDR_input(Buffer& buf):buf(buf) {}

	template <class Type>
	void visit(Type& data)
		{input_word(data);}

	void visit(float& data)
		{input_word(*reinterpret_cast<word*>(&data));}

	void visit(longlong& data)
		{input_hyper(data);}
	void visit(unsignedlonglong& data)
		{input_hyper(data);}
	void visit(double& data)
		{input_hyper(*reinterpret_cast<longlong*>(&data));}

	template <class CharSet>
	void visit(char* data, int size, CharSet) {
		buf.read(data, size);
		if (size%4!=0)
			buf.require(4-size%4);
	}
};

template <class Stream>
class XDR_input: public _XDR_input<_istream_adaptor<Stream> > {
 private:
 	_istream_adaptor<Stream> adaptor;

 public:
 	XDR_input(Stream& str):adaptor(str),
		_XDR_input<_istream_adaptor<Stream> >(adaptor) {}
};
  
template <>
class XDR_input<buffer>: public _XDR_input<buffer> {
 public:
 	XDR_input(buffer& buf):_XDR_input<buffer>(buf) {}
};
  
};

#endif
