/*
 * Compatibility with C++ standard library
 *
 * Copyright (C) 1998-2001 Jose' Orlando Pereira, Universidade do Minho
 */
/* XTL2 - the eXternalization Template Library v. 2
 * http://xtl.sourceforge.net
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 *
 * $Id: stlcompat.h,v 1.5 2001/01/04 11:12:53 jop Exp $
 */


#ifndef __XTL2_STLCOMPAT_H
#define __XTL2_STLCOMPAT_H

namespace xtl {

// adaptor

template <class Type>
struct _container_adaptor {
	Type& data;

	_container_adaptor(Type& d):data(d) {}

	template <class Visitor>
	void composite(Visitor& visitor, destructor* kind) {
		visitor.visit(data.size());
		for(typename Type::const_iterator i=data.begin();
			i!=data.end();
			i++)
			visitor>>*i;
	}

	template <class Visitor>
	void composite(Visitor& visitor, constructor* kind) {
		int size;
		visitor.visit(size);
		for(int i=0;i<size;i++) {
			typename Type::value_type v;
			visitor>>v;
			data.insert(data.end(), v);
		}
	}
};

template <class Type>
inline _container_adaptor<Type> container(Type& p) {
	return _container_adaptor<Type>(p);
}

struct _string_adaptor {
	std::string& data;

	_string_adaptor(std::string& d):data(d) {}

	template <class Visitor>
	void composite(Visitor& visitor, destructor* kind) {
		int size=data.size();
		visitor.visit(size);
		visitor.visit(data.data(), size,(ascii*)0);
	}

	template <class Visitor>
	void composite(Visitor& visitor, constructor* kind) {
		 int size;
		 visitor.visit(size);
		 char* ptr=new char[size+1];
		 visitor.visit(ptr, size,(ascii*)0);
		 ptr[size]=0;
		 data=std::string(ptr,size);
	}
};

};

#include <utility>
#include <vector>
#include <list>
#include <map>

template <class Visitor, class Kind, class Type1, class Type2>
inline void visit(Visitor& visitor, Kind kind, std::pair<Type1,Type2>& data) {
	visitor>>data.first;
	visitor>>data.second;
}

template <class Visitor, class Kind, class Type>
inline void visit(Visitor& visitor, Kind kind, std::list<Type>& data) {
	visitor>>xtl::container(data);
}

template <class Visitor, class Kind, class Type>
inline void visit(Visitor& visitor, Kind kind, std::vector<Type>& data) {
	visitor>>xtl::container(data);
}

template <class Visitor, class Kind, class Type1, class Type2>
inline void visit(Visitor& visitor, Kind kind, std::map<Type1,Type2>& data) {
	visitor>>xtl::container(data);
}

#include <string>

template <class Visitor, class Kind>
inline void visit(Visitor& visitor, Kind kind, std::string& data) {
	xtl::_string_adaptor(data).composite(visitor,kind);
}

#endif

